import * as THREE from "three";
import { useEffect, useRef } from "react";
import { useThree } from "@react-three/fiber";
import * as turf from "@turf/turf";
import * as Types from "../../../types/world/worldTypes";
// import { Feature, Polygon, MultiPolygon } from "@turf/helpers";
type Point = {
  position: { x: number; y: number; z: number };
  uuid: string;
};

type LineData = {
  type: string;
  line: Point[];
};

type PolygonGeneratorProps = {
  processPoint: LineData[];
  groupRef: React.RefObject<THREE.Group>;
};

export default function PolygonGenerator({
  processPoint,
  groupRef,
}: PolygonGeneratorProps) {
  const { scene } = useThree();
  let lines : Types.Lines;
//   console.log("storedFloorItems: ", lines);

//   useEffect(() => {
//     if (!groupRef?.current) return;
//     const groundGeometry = new THREE.BoxGeometry(80, 1, 60);
//     const groundMaterial = new THREE.MeshBasicMaterial({ color: "#99ff99" });
//     const groundMesh = new THREE.Mesh(groundGeometry, groundMaterial);
//     groundMesh.position.set(-10, -0.5, 3);
//     groundMesh.name = "GroundMesh";
//     groupRef.current.add(groundMesh);
//     scene.add(groupRef.current);
//   }, [groupRef, scene]);

  useEffect(() => {
    if (!processPoint) return;
    const wallInLayer = processPoint?.filter(
      (line) => line.type === "WallLine"
    );
    const wallPoints = wallInLayer.map((pair) =>
      pair?.line.map((vals) => vals.position)
    );
    renderWallGeometry(wallPoints);

    const linesInLayer = processPoint?.filter(
      (line) => line.type === "AisleLine"
    );
    const result = linesInLayer.map((pair) =>
      pair?.line.map((point) => ({
        position: [point.position.x, point.position.z],
        uuid: point.uuid,
      }))
    );

    const lineFeatures = result.map((line) =>
      turf.lineString(line.map((p) => p.position))
    );
    const polygons = turf.polygonize(turf.featureCollection(lineFeatures));
    let union: any[] = [];

    polygons.features.forEach((feature) => {
      union.push(feature);
    });

    if (union.length > 0) {
      const unionResult = turf.union(turf.featureCollection(union));
      if (unionResult && unionResult.geometry.type === "MultiPolygon") {
        unionResult.geometry.coordinates.forEach((poly) => {
          const Coordinates = poly[0].map(
            ([x, z]) => new THREE.Vector3(x, 0, z)
          );
          renderBoxGeometry(Coordinates);
        });
      } else if (unionResult && unionResult.geometry.type === "Polygon") {
        const Coordinates = unionResult.geometry.coordinates[0].map(
          ([x, z]) => new THREE.Vector3(x, 0, z)
        );
        renderBoxGeometry(Coordinates);
      }
    }
  }, [processPoint]);

  const renderBoxGeometry = (coordinates: THREE.Vector3[]) => {
    const minX = Math.min(...coordinates.map((p) => p.x));
    const maxX = Math.max(...coordinates.map((p) => p.x));
    const minZ = Math.min(...coordinates.map((p) => p.z));
    const maxZ = Math.max(...coordinates.map((p) => p.z));

    const width = maxX - minX;
    const depth = maxZ - minZ;
    const height = 3;

    const geometry = new THREE.BoxGeometry(width, height, depth);
    const material = new THREE.MeshBasicMaterial({ color: "#ff66cc" });
    const mesh = new THREE.Mesh(geometry, material);
    mesh.position.set((minX + maxX) / 2, height / 2, (minZ + maxZ) / 2);

    groupRef?.current?.add(mesh);
    scene.add(groupRef.current!);
  };

  const renderWallGeometry = (
    walls: { x: number; y: number; z: number }[][]
  ) => {
    walls.forEach((wall) => {
      if (wall.length < 2) return;

      for (let i = 0; i < wall.length - 1; i++) {
        const start = new THREE.Vector3(wall[i].x, wall[i].y, wall[i].z);
        const end = new THREE.Vector3(
          wall[i + 1].x,
          wall[i + 1].y,
          wall[i + 1].z
        );
        const wallHeight = 10;
        const direction = new THREE.Vector3().subVectors(end, start);
        const length = direction.length();
        direction.normalize();

        const wallGeometry = new THREE.BoxGeometry(length, wallHeight, 0.5);
        const wallMaterial = new THREE.MeshBasicMaterial({
          color: "#aaa",
          transparent: true,
          opacity: 0.5,
        });
        const wallMesh = new THREE.Mesh(wallGeometry, wallMaterial);

        const midPoint = new THREE.Vector3()
          .addVectors(start, end)
          .multiplyScalar(0.5);
        wallMesh.position.set(midPoint.x, wallHeight / 2, midPoint.z);

        const quaternion = new THREE.Quaternion();
        quaternion.setFromUnitVectors(new THREE.Vector3(1, 0, 0), direction);
        wallMesh.quaternion.copy(quaternion);

        groupRef?.current?.add(wallMesh);
        scene.add(groupRef.current!);

        const lineGeometry = new THREE.BufferGeometry().setFromPoints([
          start,
          end,
        ]);
        const lineMaterial = new THREE.LineBasicMaterial({ color: "blue" });
        const line = new THREE.Line(lineGeometry, lineMaterial);

        groupRef?.current?.add(line);
      }
    });
  };

  return null;
}
